@@ -89,5 +89,6 @@ internalPattern["GBID_CPSD_PEP"] = "[A-Z][a-z]{4,7}[0-9]+(-[A-Za-z0-9]*)+"
 internalPattern["GBID_CPSD_ORG"] = "[0-9-.]+"
 #internalPattern["GBID_CPSD_GENE"] = "" yet to be defined
 #internalPattern["GBID_CPSD_TRANS"] = "" yet to be defined
-internalPattern["GBID_CPSD_VERSION"] = "^20[0-9]{2}\.[0]$"
+internalPattern["GBID_CPSD_VERSION"] = "^20[0-9]{2}\.[0-9]$"
 #internalPattern["GBID_MOA_TARGET"] = "" yet to be defined
+internalPattern["GBID_CPSD_GROUP"] = "Group-[A-Za-z-]+-20[0-9]{2}-[0-9]"
