@@ -20,14 +20,14 @@ class NtWriter:
 		self.f.close()
 		self.f = None
 		if exType is None:
-			print ("Renaming to: " + self.fname) 
+			#print ("Renaming to: " + self.fname) 
 			os.rename(self.tmpname, self.fname)
 		else:
-			print ("Deleting temp file")
+			#print ("Deleting temp file")
 			os.unlink (self.tmpname)
 
 
-	def __init__ (self, fname = None, shapes = None, validate=True, escapeUnicode=True):
+	def __init__ (self, fname = None, shapes = None, writeType = "wb", validate=True, escapeUnicode=True):
 		r"""
 		fname:
 			if no filename is passed, the triples are written to standard output.
@@ -53,15 +53,15 @@ class NtWriter:
 			if fname.endswith (".gz"):
 				# can't open temp file to gzip, see http://www.enricozini.org/2011/cazzeggio/python-gzip/
 				self.tmpname = tempfile.mktemp(prefix = "ntwriter-", suffix = ".tmp", dir = parentdir)
-				fh = gzip.open (self.tmpname, "w")
+				fh = gzip.open (self.tmpname, writeType)
 			else:
 				# this is the proper, race-less way to atomically create and open a temporary file
 				# Unfortunately it only works for non-gz files
 				(handle, self.tmpname) = tempfile.mkstemp(dir = parentdir)
-				fh = os.fdopen(handle, "w")
+				fh = os.fdopen(handle, writeType)
 				#fh = open (fname, "w")
 			
-		print ("Using temp file: " + self.tmpname)
+		#print ("Using temp file: " + self.tmpname)
 
 		if self.escapeUnicode: writer = codecs.getwriter("ascii")
 		else: writer = codecs.getwriter("utf-8")
@@ -171,5 +171,5 @@ class NtWriter:
 	def close(self):
 		self.f.close ()
 		self.f = None
-		print ("Renaming to: " + self.fname)
+		#print ("Renaming to: " + self.fname)
 		os.rename(self.tmpname, self.fname)
